#!/usr/bin/env python

"""
####################################################
# ba_hologram.py #   https://gimplearn.net/viewtopic.php?f=3&t=1441
#################################################### 
Brian Andersen's Interpretation of tutorial found here:
 https://www.eurobricks.com/forum/index.php?/forums/topic/102302-tutorial-picture-editing-create-star-wars-holograms/

 ...................................................... 
                            
####################
# COPYRIGHT NOTICE #
###############################################################################
#                                                                             #
# This program is free software you can redistribute it and/or modify         #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation either version 3 of the License, or            #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY without even the implied warranty of               #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program if not, you can view the GNU General Public         #
# License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html  #
# Alternatively you can write to the Free Software Foundation, Inc., 675 Mass #
# Ave, Cambridge, MA 02139, USA.                                              #
###############################################################################
"""
# MrQ 2020-03-27 add option: active layer or all layers

from gimpfu import *
import colorsys
# ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## #
#########################################################################################################################################################################################################################
def ba_hologram (image ,layer ,layerg, color): #########
#########################################################################################################################################################################################################################
# ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## # # ## ### #### ### ## # ## ### #### ### ## #
## test for open RGB mode image - script intended for working with animated gifs - won't run on Index type image.
    if not image:
        pdb.gimp_message( 'Script needs an open RGB type image.' )
    else:  
        imagetype = pdb.gimp_image_base_type(image)
        if imagetype != 0:
            pdb.gimp_message( "Wrong Image Type: Use IMAGE > MODE to set image to RGB type." )
            return ()

## add alpha channel 
    pdb.gimp_layer_add_alpha(layer)

## set behaviour
    pdb.gimp_image_undo_group_start(image)
    pdb.gimp_context_push()
    pdb.gimp_context_set_foreground('#000000')
    pdb.gimp_context_set_background('#FFFFFF')

## get size
    n_wid = image.width
    n_hei = image.height

## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
    if layerg==0:
		pdb.gimp_image_set_active_layer(image,layer)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		display = pdb.gimp_display_new(new_image)
		item_copy = pdb.gimp_layer_new_from_drawable(layer, new_image)
		pdb.gimp_image_insert_layer(new_image, item_copy, None, 0)
		item_copy.name = layer.name
		pdb.gimp_layer_add_alpha(layer)

		pdb.gimp_item_set_visible(layer,True)
		holo_lay = pdb.gimp_layer_new_from_drawable(layer, new_image)
		new_image.add_layer (holo_lay, 0)
		holo_lay.opacity = 75.0 
		holo_lay.name = "hologram"
		holo_lay.mode = LAYER_MODE_NORMAL_LEGACY

		#color = pdb.gimp_context_get_foreground()


		h,l,s = colorsys.rgb_to_hls(color[0]/255.0,color[1]/255.0,color[2]/255.0)
		hue = h * 360.0
		saturation = s * 100.0
		lightness = l * 200.0 - 100.0

	
		#pdb.gimp_colorize(holo_lay,227,58,30)  
		pdb.gimp_drawable_colorize_hsl(holo_lay, hue, saturation, lightness)

		stripe_lay = pdb.gimp_layer_new_from_drawable(layer, new_image)
		new_image.add_layer(stripe_lay,0)
		stripe_lay.name = 'stripes'
		stripe_lay.mode = LAYER_MODE_OVERLAY_LEGACY
		pdb.gimp_image_select_item(new_image, 2, stripe_lay)
		pdb.gimp_edit_clear(stripe_lay)
		pdb.gimp_image_select_item(new_image,2,holo_lay)
		sx = ex = int(n_wid/2)
		ey = 0
		sy = int(n_hei*.02)
		pdb.gimp_edit_blend(stripe_lay,0,0,0,100,0,REPEAT_TRIANGULAR,0,0,3,0.2,FALSE,sx,sy,ex,ey)

		steps = int((n_wid+n_hei)*.005)
		glow_lay = holo_lay.copy()
		new_image.add_layer(glow_lay,0)
		pdb.gimp_image_lower_item_to_bottom(new_image,glow_lay)
		glow_lay.name = 'glow'
		newname=layer.name
		pdb.gimp_selection_grow(new_image, steps)  
		pdb.gimp_context_set_foreground(color)
		pdb.gimp_edit_fill(glow_lay,0)
		pdb.gimp_selection_none(new_image)
		blursize = int(steps*10)
		pdb.plug_in_gauss(new_image, glow_lay, blursize, blursize,1) #RLE = 1
		#layer = pdb.gimp_image_merge_visible_layers(new_image,0)
		#layer.name = newname
		pdb.gimp_item_set_visible(layer,True)	
    if layerg==1:
		layerlist = image.layers
		for layer in layerlist:
			pdb.gimp_layer_add_alpha(layer)
			pdb.gimp_item_set_visible(layer,False)
			
		for layer in layerlist:			
			pdb.gimp_item_set_visible(layer,True)
			holo_lay = layer.copy()
			image.add_layer (holo_lay, 0)
			holo_lay.opacity = 75.0 
			holo_lay.name = "hologram"
			holo_lay.mode = LAYER_MODE_NORMAL_LEGACY
			h,l,s = colorsys.rgb_to_hls(color[0]/255.0,color[1]/255.0,color[2]/255.0)
			hue = h * 360.0
			saturation = s * 100.0
			lightness = l * 200.0 - 100.0			
			#pdb.gimp_colorize(holo_lay,227,58,30)
			pdb.gimp_drawable_colorize_hsl(holo_lay, hue, saturation, lightness)			


			stripe_lay = layer.copy()
			image.add_layer(stripe_lay,0)
			stripe_lay.name = 'stripes'
			stripe_lay.mode = LAYER_MODE_OVERLAY_LEGACY
			pdb.gimp_image_select_item(image, 2, stripe_lay)
			pdb.gimp_edit_clear(stripe_lay)
			pdb.gimp_image_select_item(image,2,holo_lay)
			sx = ex = int(n_wid/2)
			ey = 0
			sy = int(n_hei*.02)
			pdb.gimp_edit_blend(stripe_lay,0,0,0,100,0,REPEAT_TRIANGULAR,0,0,3,0.2,FALSE,sx,sy,ex,ey)

			steps = int((n_wid+n_hei)*.005)
			glow_lay = holo_lay.copy()
			image.add_layer(glow_lay,0)
			pdb.gimp_image_lower_item_to_bottom(image,glow_lay)
			glow_lay.name = 'glow'
			newname=layer.name
			pdb.gimp_selection_grow(image, steps)  
			pdb.gimp_context_set_foreground(color)
			pdb.gimp_edit_fill(glow_lay,0)
			pdb.gimp_selection_none(image)
			blursize = int(steps*10)
			pdb.plug_in_gauss(image, glow_lay, blursize, blursize,1) #RLE = 1
			layer = pdb.gimp_image_merge_visible_layers(image,0)
			layer.name = newname
			pdb.gimp_item_set_visible(layer,False)
   
# ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## #
## end of main loop
# ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## #

## graceful exit

    pdb.gimp_displays_flush()
    pdb.gimp_context_pop()
    pdb.gimp_image_undo_group_end(image)

register(
    "ba_hologram",    
    "Produces hologram style forgrounds from set of input layers.\nNeeds an open RGB type image. ",   
    "",
    "Brian", 
    "Brian Andersen ", 
    "2018",
    "BA Hologram Effect...", 
    "RGB*", 
    [
      (PF_IMAGE,   "image"      ,"Input image                                  ", None),
      (PF_DRAWABLE,"drawable"   ,"Input drawable                               ", None),
      (PF_OPTION, "layerg"," Use efect for:: ", 1, [" ACTIVE LAYER ", " ALL LAYERS "]),
      (PF_COLOR,  "color"    , "FG Color [#8294d5]", (130, 148, 213)),
      #(PF_SPINNER, "hue"    , "Hue [227]",  227,   ( 0.0, 360.0, 1)),
      #(PF_SPINNER, "saturation"    , "Saturation [58]",  58,   ( 0.0, 100.0, 1)),
      #(PF_SPINNER, "lightness"    , "Lightness [30]",  30,   ( 0.0, 100.0, 1)),	  
	  
    ],[],
    ba_hologram, menu = "<Image>/Filters/Render" 
    )
main()
